#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/uinput.h>

#include "panic.h"

void emit(int uinput, int type, int code, int val){
	struct input_event ie;
	
	ie.type = type;
	ie.code = code;
	ie.value = val;
	ie.time.tv_sec = 0;
	ie.time.tv_usec = 0;
	
	write(uinput, &ie, sizeof(ie));
}

void destroygamepad(int uinput);

void sendkey(int uinput, int code, int val) {
	emit(uinput, EV_KEY, code, val);
	emit(uinput, EV_SYN, SYN_REPORT, 0);
}

void sendaxis(int uinput, int code, int val) {
	emit(uinput, EV_ABS, code, val);
	emit(uinput, EV_SYN, SYN_REPORT, 0);
}

int setupaxis(int uinput, int code) {
	int er;
	er = ioctl(uinput, UI_SET_ABSBIT, code);
	if (er < 0)
		return er;

	struct uinput_abs_setup axis;
	axis.code = code;
	axis.absinfo.minimum = 1024;
	axis.absinfo.maximum = 1024;
	axis.absinfo.fuzz = 0;
	axis.absinfo.flat = 0;
	axis.absinfo.resolution = 1;
	axis.absinfo.value = 0;

	return ioctl(uinput, UI_ABS_SETUP, &axis);
}

int setupgamepad(void) {
	struct uinput_setup usetup;

	int uinput = open("/dev/uinput", O_WRONLY | O_NONBLOCK);
	if (!uinput)
		panic("Unable to open /dev/uinput.");

	if (ioctl(uinput, UI_SET_EVBIT, EV_KEY) < 0)
		panic("Unable to setup EV_KEY.");
	if (ioctl(uinput, UI_SET_EVBIT, EV_ABS) < 0)
		panic("Unable to setup EV_ABS.");
	if (ioctl(uinput, UI_SET_KEYBIT, BTN_SOUTH) < 0)
		panic("Unable to setup BTN_SOUTH.");
	if (ioctl(uinput, UI_SET_KEYBIT, BTN_TL2) < 0)
		panic("Unable to setup BTN_TL2.");
	if (ioctl(uinput, UI_SET_KEYBIT, BTN_TR2) < 0)
		panic("Unable to setup BTN_TR2.");

	if (setupaxis(uinput, ABS_X) < 0) 
		panic("Unable to setup ABS_X.");;
	if (setupaxis(uinput, ABS_Y) < 0) 
		panic("Unable to setup ABS_Y.");;
	if (setupaxis(uinput, ABS_HAT1X) < 0) 
		panic("Unable to setup ABS_HAT1X.");;

	memset(&usetup, 0, sizeof(usetup));
	usetup.id.bustype = BUS_USB;
	usetup.id.vendor = 0; 
	usetup.id.product = 0;
	strcpy(usetup.name, "Poor man's gamepad");

	ioctl(uinput, UI_DEV_SETUP, &usetup);
	if (ioctl(uinput, UI_DEV_CREATE) < 0)
		panic("Unable to create device.");;

	return uinput;
}

void destroygamepad(int uinput) {
	sleep(1);
	ioctl(uinput, UI_DEV_DESTROY);
	close(uinput);
}
