#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <time.h>
#include <sys/poll.h>
#include <unistd.h>
#include <fcntl.h>

#include <linux/uinput.h>

#include "gamepad.h"
#include "panic.h"


int waitmouse(int mouse, struct input_event* ie) {
	while (1) {
		struct pollfd fds[1];
		fds[0].fd = mouse;
		fds[0].events = POLLIN;
		poll(fds, 1, 0);
		if (fds[0].revents & POLLIN) {
			read(mouse, ie, sizeof(struct input_event));
			if (ie->type != EV_SYN)
				return 1;
		} else
			return 0;
	}
}

int clamp(int num) {
	// Bruh
	if (num > 33000)
		return 33000;
	if (num < -33000)
		return -33000;
	return num;
}

int cut(int num, int amount) {
	if (num > -amount && num < amount)
		return 0;
	return num;
}

int drop(int num, int amount) {
	if (num > 0)
		return num - amount;
	if (num < 0)
		return num + amount;
	return 0;
}

int main(int argc, char* argv[]) {
	if (argc == 0) {
		fprintf(stderr, "????\n");
		return 255;
	}
	if (argc != 11) {
		printf("Usage: %s devicepath "
			       	"sensitivity_horizontal sensitivity_vertical sensitivity_wheel " 
				"deadzone_horizontal deadzone_vertical deadzone_wheel "
				"drop_horizontal drop_vertical drop_wheel\n", argv[0]);
		return 1;
	}
	struct input_event ie;
	int axis_x = 0;
	int axis_y = 0;
	int axis_w = 0;
	int scale_axis_x = strtol(argv[2], NULL, 10);
	int scale_axis_y = strtol(argv[3], NULL, 10);
	int scale_axis_w = strtol(argv[4], NULL, 10);
	int deadzone_axis_x = strtol(argv[5], NULL, 10);
	int deadzone_axis_y = strtol(argv[6], NULL, 10);
	int deadzone_axis_w = strtol(argv[7], NULL, 10);
	int drop_axis_x = strtol(argv[8], NULL, 10);
	int drop_axis_y = strtol(argv[9], NULL, 10);
	int drop_axis_w = strtol(argv[10], NULL, 10);

	char* device = argv[1];
	int mouse = open(device, 0);
	if (!mouse)
		panic("Unable to open device.");

	int gamepad = setupgamepad();

	struct timespec prev, current;
	clock_gettime(CLOCK_MONOTONIC, &prev);
	while (1) {
		clock_gettime(CLOCK_MONOTONIC, &current);
		long delta = (current.tv_sec - prev.tv_sec) * 1000000000 + current.tv_nsec - prev.tv_nsec;
		if (delta > 10000000) {
			axis_x = drop(axis_x, drop_axis_x);
			axis_y = drop(axis_y, drop_axis_y);
			axis_w = drop(axis_w, drop_axis_w);
			prev.tv_nsec = current.tv_nsec;
			prev.tv_sec = current.tv_sec;
		}
		if (waitmouse(mouse, &ie)) {
			if (ie.type == EV_REL)
				switch (ie.code) {
					case REL_X:
						axis_x = axis_x + ie.value * scale_axis_x;
						break;
					case REL_Y:
						axis_y = axis_y + ie.value * scale_axis_y;
						break;
					case REL_WHEEL:
						axis_w = axis_w + ie.value * scale_axis_w;
						break;
				}
			axis_x = clamp(axis_x);
			axis_y = clamp(axis_y);
			axis_w = clamp(axis_w);
		}
		sendaxis(gamepad, ABS_X,     cut(axis_x, deadzone_axis_x));
		sendaxis(gamepad, ABS_Y,     cut(axis_y, deadzone_axis_y));
		sendaxis(gamepad, ABS_HAT1X, cut(axis_w, deadzone_axis_w));
		if (ie.type == EV_KEY && ie.code == BTN_LEFT)
			sendkey(gamepad, BTN_TL2, ie.value);
		if (ie.type == EV_KEY && ie.code == BTN_MIDDLE)
			sendkey(gamepad, BTN_SOUTH, ie.value);
		if (ie.type == EV_KEY && ie.code == BTN_RIGHT)
			sendkey(gamepad, BTN_TR2, ie.value);
	}
	return 0;
}
